import { Note, Notepad, Trie } from './index';
import { EncryptionMethod } from './crypto';
export declare type FlatNotepadOptions = {
    lastModified?: Date;
    notepadAssets?: string[];
    sections?: {
        [internalRef: string]: FlatSection;
    };
    notes?: {
        [internalRef: string]: Note;
    };
    crypto?: EncryptionMethod;
};
export declare type FlatSection = {
    title: string;
    internalRef: string;
    parentRef?: string;
};
/**
 * A FlatNotepad is similar to the {@link Notepad} class, but it stores all the notes/sections
 * as in flat structures. FlatNotepads will likely be better for internal use in many situations.
 *
 * Something to remember is that all operations on this class like addSection, will return a <strong>new</strong>
 * object of this class, and not modify the existing one.
 */
export default class FlatNotepad {
    readonly title: string;
    readonly lastModified: string;
    readonly sections: {
        [internalRef: string]: FlatSection;
    };
    readonly notes: {
        [internalRef: string]: Note;
    };
    readonly notepadAssets: string[];
    readonly crypto?: EncryptionMethod;
    constructor(title: string, opts?: FlatNotepadOptions);
    static makeFlatSection(title: string, parentRef?: string): FlatSection;
    addSection(section: FlatSection): FlatNotepad;
    addNote(note: Note): FlatNotepad;
    addAsset(uuid: string): FlatNotepad;
    modified(lastModified?: Date): FlatNotepad;
    /**
     * Unlike the {@link Notepad}, this uses an indexed lookup system. This should be faster than a
     * traditional notepad search.
     *
     * @param {Trie} trie The search trie for the notepad
     * @param {string} query Can either be a title-search or a hashtag-search
     * @returns {Note[]}
     */
    search(trie: Trie, query: string): Note[];
    /**
     * This will convert everything into the formal {@link Notepad} structure, however no {@link Asset}s will
     * be restored. The client should rebuild the assets after this using the values in {@link notepadAssets}
     * @returns {Notepad}
     */
    toNotepad(): Notepad;
    clone(opts?: Partial<FlatNotepadOptions>, title?: string): FlatNotepad;
    pathFrom(obj: Note | FlatSection): (FlatSection | FlatNotepad)[];
}
