"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("./index");
const date_fns_1 = require("date-fns");
const date_formats_1 = require("./date-formats");
/**
 * A FlatNotepad is similar to the {@link Notepad} class, but it stores all the notes/sections
 * as in flat structures. FlatNotepads will likely be better for internal use in many situations.
 *
 * Something to remember is that all operations on this class like addSection, will return a <strong>new</strong>
 * object of this class, and not modify the existing one.
 */
class FlatNotepad {
    constructor(title, opts = {}) {
        this.title = title;
        this.lastModified = date_fns_1.format(opts.lastModified || new Date(), date_formats_1.LAST_MODIFIED_FORMAT);
        this.sections = opts.sections || {};
        this.notes = opts.notes || {};
        this.notepadAssets = opts.notepadAssets || [];
        if (opts.crypto)
            this.crypto = opts.crypto;
    }
    static makeFlatSection(title, parentRef) {
        const tmpSection = new index_1.Section(title);
        return { title: tmpSection.title, internalRef: tmpSection.internalRef, parentRef };
    }
    addSection(section) {
        return this.clone({
            sections: Object.assign(Object.assign({}, this.sections), { [section.internalRef]: section })
        });
    }
    addNote(note) {
        // Ensure our parent is just a string for the section's internalRef, not the whole Parent object
        if (typeof note.parent !== 'string') {
            note = note.clone({
                parent: note.parent.internalRef
            });
        }
        return this.clone({
            notes: Object.assign(Object.assign({}, this.notes), { [note.internalRef]: note })
        });
    }
    addAsset(uuid) {
        return this.clone({
            notepadAssets: [
                ...this.notepadAssets,
                uuid
            ]
        });
    }
    modified(lastModified = new Date()) {
        return this.clone({
            lastModified
        });
    }
    /**
     * Unlike the {@link Notepad}, this uses an indexed lookup system. This should be faster than a
     * traditional notepad search.
     *
     * @param {Trie} trie The search trie for the notepad
     * @param {string} query Can either be a title-search or a hashtag-search
     * @returns {Note[]}
     */
    search(trie, query) {
        return index_1.Trie.search(trie, query).map(ref => this.notes[ref]);
    }
    /**
     * This will convert everything into the formal {@link Notepad} structure, however no {@link Asset}s will
     * be restored. The client should rebuild the assets after this using the values in {@link notepadAssets}
     * @returns {Notepad}
     */
    toNotepad() {
        const buildSection = (flat) => {
            let section = new index_1.Section(flat.title, [], [], flat.internalRef);
            // Restore sub-sections
            Object.values(this.sections)
                .filter(s => s.parentRef === flat.internalRef)
                .map(s => section = section.addSection(buildSection(s)));
            // Restore notes
            Object.values(this.notes)
                .filter(n => n.parent === flat.internalRef)
                .map(n => section = section.addNote(n));
            return section;
        };
        let notepad = new index_1.Notepad(this.title, {
            lastModified: date_fns_1.parse(this.lastModified, date_formats_1.LAST_MODIFIED_FORMAT, new Date()),
            notepadAssets: this.notepadAssets,
            crypto: this.crypto
        });
        // Add all the sections + notes
        Object.values(this.sections)
            .filter(s => !s.parentRef)
            .forEach(s => notepad = notepad.addSection(buildSection(s)));
        return notepad;
    }
    clone(opts = {}, title = this.title) {
        return new FlatNotepad(title, Object.assign({ lastModified: date_fns_1.parse(this.lastModified, date_formats_1.LAST_MODIFIED_FORMAT, new Date()), sections: this.sections, notes: this.notes, notepadAssets: this.notepadAssets, crypto: this.crypto }, opts));
    }
    pathFrom(obj) {
        const parents = [];
        if (!!obj.parent) { // This is a note
            obj = this.sections[obj.parent];
        }
        else {
            const parent = obj.parentRef;
            if (!parent)
                return [this];
            obj = this.sections[obj.parentRef];
        }
        let tmp = obj;
        while (true) {
            parents.unshift(tmp);
            if (!tmp.parentRef)
                return [this, ...parents];
            tmp = this.sections[tmp.parentRef];
        }
    }
}
exports.default = FlatNotepad;
//# sourceMappingURL=FlatNotepad.js.map