"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.canOptimiseElement = void 0;
const NPXObject_1 = require("./NPXObject");
const date_fns_1 = require("date-fns");
const date_formats_1 = require("./date-formats");
class Note extends NPXObject_1.NPXObject {
    constructor(title, time = new Date().getTime(), elements = [], bibliography = [], internalRef, parent) {
        super(title, internalRef, parent);
        this.title = title;
        this.time = time;
        this.elements = elements;
        this.bibliography = bibliography;
    }
    addElement(element) {
        return this.clone({
            elements: [
                ...this.elements,
                element
            ]
        });
    }
    addSource(source) {
        return this.clone({
            bibliography: [
                ...this.bibliography,
                source
            ]
        });
    }
    search(query) {
        // Title search
        let pattern = new RegExp("\\b" + query.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, '\\$&'), 'i');
        if (pattern.test(this.title))
            return [this];
        // Hashtag search
        if (query.length > 1 && query.charAt(0) === '#') {
            pattern = new RegExp("(^|\\s)" + query.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&") + "(\\b)", 'i');
            // Check if any markdown elements contain the hashtag
            if (this.elements
                .filter(e => e.type === 'markdown')
                .some(e => pattern.test(e.content)))
                return [this];
        }
        return [];
    }
    getHashtags() {
        return this.elements
            .filter(e => e.type === 'markdown')
            .map(e => e.content.match(/(^|\s)(#[a-z\d-]+)/gi))
            .filter(res => !!res)
            .map(matches => Array.from(matches))
            .reduce((acc, val) => acc.concat(val), [])
            .map(hashtag => hashtag.trim());
    }
    getHeadingWords() {
        return new Set(this.elements
            .filter(e => e.type === 'markdown')
            .flatMap(e => Array.from(e.content.matchAll(/^#{1,3} (.+)$/gmi)))
            .flatMap(matches => matches[1].trim().split(' ')));
    }
    toXmlObject() {
        const elements = {};
        this.elements.forEach(e => {
            if (!elements[e.type])
                elements[e.type] = [];
            elements[e.type].push({
                $: Object.assign({}, e.args),
                _: e.content
            });
        });
        const bibliography = this.bibliography.map(source => {
            return {
                $: {
                    id: source.id,
                    item: source.item
                },
                _: source.content
            };
        });
        return {
            note: Object.assign({ $: {
                    title: this.title,
                    time: date_fns_1.format(new Date(this.time), date_formats_1.LAST_MODIFIED_FORMAT)
                }, addons: [[]], bibliography: (bibliography.length > 0) ? { source: bibliography } : [[]] }, elements)
        };
    }
    async toMarkdown(assets) {
        const assetMap = {};
        assets.forEach(a => assetMap[a.uuid] = a);
        const md = (await Promise.all(this.elements
            .filter(e => ['markdown', 'drawing', 'image'].includes(e.type))
            .map(async (e) => {
            let md;
            switch (e.type) {
                case 'markdown':
                    md = e.content + '\n\n';
                    break;
                case 'drawing':
                case 'image':
                    const asset = assetMap[e.args.ext || 0];
                    if (!asset)
                        return '';
                    md = `![](${await asset.toString()})\n\n`;
                    break;
            }
            if (!md)
                return '';
            // Bibliography
            const bib = this.bibliography
                .filter(s => s.item === e.args.id)
                .map(s => s.content);
            if (bib.length > 0) {
                md += `***Bibliography***  \n`;
                md += bib
                    .map(content => `- <${content}>\n`)
                    .reduce((str, source) => str += source, '') + '\n';
            }
            return md;
        })))
            .reduce((str, elementMd) => str += elementMd, '');
        return { title: this.title, md };
    }
    clone(opts = {}) {
        return new Note(opts.title || this.title, opts.time || this.time, [
            ...(opts.elements || this.elements),
        ], [
            ...(opts.bibliography || this.bibliography),
        ], opts.internalRef || this.internalRef, opts.parent || this.parent);
    }
}
exports.default = Note;
function canOptimiseElement(el) {
    var _a;
    const isEligible = (_a = el.args.canOptimise) !== null && _a !== void 0 ? _a : el.type === 'image';
    // Only fixed sizes can be optimised
    const width = parseInt(el.args.width, 10);
    const height = parseInt(el.args.height, 10);
    if (isNaN(width) || isNaN(height))
        return false;
    return isEligible;
}
exports.canOptimiseElement = canOptimiseElement;
//# sourceMappingURL=Note.js.map