"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const date_fns_1 = require("date-fns");
const _1 = require("./");
const xml2js_1 = require("xml2js");
const crypto_1 = require("./crypto");
const date_formats_1 = require("./date-formats");
/**
 * This class is identical in structure to the old Notepad class from the original parser.
 * This represents the notepad as a tree. If you're looking for a flatter structure you can use {@link FlatNotepad}.
 *
 * Something to remember is that all operations on this class like addSection, will return a <strong>new</strong>
 * object of this class, and not modify the existing one.
 */
class Notepad {
    constructor(title, opts = {}) {
        this.title = title;
        this.lastModified = date_fns_1.format(opts.lastModified || new Date(), date_formats_1.LAST_MODIFIED_FORMAT);
        this.sections = opts.sections || [];
        this.notepadAssets = opts.notepadAssets || [];
        this.assets = opts.assets || [];
        if (opts.crypto)
            this.crypto = opts.crypto;
    }
    addSection(section) {
        const notepad = this.clone({
            sections: [
                ...this.sections,
                section
            ]
        });
        section.parent = notepad;
        return notepad;
    }
    addAsset(asset) {
        return this.clone({
            assets: [
                ...this.assets,
                asset
            ],
            notepadAssets: [
                ...this.notepadAssets,
                asset.uuid
            ]
        });
    }
    /**
     * This updates the lastModified date on the notepad. This date is used for syncing so it's important
     * to call this method whenever a change is made that will need to be synced.
     * @param {Date} lastModified
     * @returns {Notepad}
     */
    modified(lastModified = new Date()) {
        return this.clone({
            lastModified
        });
    }
    /**
     * @param {string} query Can either be a title-search or a hashtag-search
     * @returns {Note[]}
     */
    search(query) {
        return this.sections
            .map(s => s.search(query))
            .reduce((acc, val) => acc.concat(val), []);
    }
    async toJson(passkey) {
        let notepad = Object.assign(Object.assign({}, this), { assets: undefined, 
            // If we're given a passkey but encryption hasn't been setup on the notepad, set it up
            crypto: (!this.crypto && !!passkey) ? 'AES-256-GZ' : this.crypto });
        let notepadToStringify = notepad;
        if (!!notepad.crypto && !!passkey)
            notepadToStringify = await crypto_1.encrypt(notepad, passkey);
        if (!!notepad.crypto && !passkey) {
            notepadToStringify = Object.assign(Object.assign({}, notepadToStringify), { crypto: undefined });
        }
        return JSON.stringify(notepadToStringify, (key, value) => {
            return (key === 'parent') ? undefined : value;
        });
    }
    async toXml() {
        const builder = new xml2js_1.Builder({
            cdata: true,
            renderOpts: {
                'pretty': false
            },
            xmldec: {
                version: '1.0',
                encoding: 'UTF-8',
                standalone: false
            }
        });
        // Generate the XML
        const obj = await this.toXmlObject();
        return builder.buildObject(obj).replace(/&#xD;/g, '');
    }
    flatten() {
        let notepad = new _1.FlatNotepad(this.title, {
            lastModified: date_fns_1.parse(this.lastModified, date_formats_1.LAST_MODIFIED_FORMAT, new Date()),
            notepadAssets: this.notepadAssets,
            crypto: this.crypto
        });
        const flattenSection = (section) => {
            let flat = { title: section.title, internalRef: section.internalRef };
            if (section.parent)
                flat.parentRef = section.parent.internalRef;
            // Add this flat section
            notepad = notepad.addSection(flat);
            section.notes.forEach(n => notepad = notepad.addNote(n));
            // Add all of its children recursively
            section.sections.forEach(s => flattenSection(s));
        };
        this.sections.forEach(s => flattenSection(s));
        return notepad;
    }
    async toMarkdown(assets) {
        return (await Promise.all(this.sections.map(s => s.toMarkdown(assets)))).reduce((acc, val) => acc.concat(val), []);
    }
    clone(opts = {}, title = this.title) {
        return new Notepad(title, Object.assign({ lastModified: date_fns_1.parse(this.lastModified, date_formats_1.LAST_MODIFIED_FORMAT, new Date()), sections: [...this.sections], notepadAssets: [...this.notepadAssets], assets: [...this.assets], crypto: this.crypto }, opts));
    }
    async toXmlObject() {
        return {
            notepad: {
                $: {
                    'xsi:noNamespaceSchemaLocation': 'https://getmicropad.com/schema.xsd',
                    title: this.title,
                    lastModified: this.lastModified,
                    'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance'
                },
                assets: [
                    {
                        asset: await Promise.all(this.assets.map(a => a.toXmlObject()))
                    }
                ],
                section: this.sections.map(s => s.toXmlObject().section)
            }
        };
    }
}
exports.default = Notepad;
//# sourceMappingURL=Notepad.js.map