"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Trie = void 0;
class Trie {
    constructor(lastModified = new Date()) {
        this._size = 0;
        this.hashtags = {};
        this.root = new TrieNode();
        this.lastModified = lastModified;
    }
    static buildTrie(notes, date = new Date()) {
        const trie = new Trie(date);
        Object.entries(notes).forEach(([noteRef, note]) => {
            // Add the note title
            const title = note.title.replace(/[()]/, '');
            title.split(/[\s\/\\,]/).forEach(word => Trie.add(trie, word, noteRef));
            // More explicit matching on headings than titles to limit false-positives
            note.getHeadingWords().forEach(headingWord => Trie.add(trie, headingWord, noteRef));
            note.getHashtags().forEach(hashtag => Trie.add(trie, hashtag, noteRef));
        });
        return trie;
    }
    static shouldReindex(trie, lastModified, numberOfNotes) {
        return lastModified.getTime() > trie.lastModified.getTime() || numberOfNotes !== trie._size;
    }
    static add(trie, key, ref) {
        key = key.toLowerCase();
        if (key.charAt(0) === '#') {
            const notes = trie.hashtags[key] || [];
            if (notes.indexOf(ref) !== -1)
                return;
            trie.hashtags[key] = [...notes, ref];
            return;
        }
        const keyChars = [...key];
        let node = trie.root;
        for (let ch of keyChars) {
            if (!node.children[ch])
                node.children[ch] = new TrieNode(ch);
            node = node.children[ch];
        }
        node.notes.push(ref);
        trie._size++;
    }
    static search(trie, query) {
        query = query.toLowerCase();
        if (query.charAt(0) === '#') {
            return trie.hashtags[query] || [];
        }
        const keyChars = [...query];
        let node = trie.root;
        for (let ch of keyChars) {
            if (!node.children[ch])
                return [];
            node = node.children[ch];
        }
        return [...new Set(node.getAllFrom())];
    }
    get size() {
        return this._size;
    }
    get availableHashtags() {
        return Object.keys(this.hashtags);
    }
    /**
     * @deprecated Use static methods instead
     */
    shouldReindex(lastModified, numberOfNotes) {
        return Trie.shouldReindex(this, lastModified, numberOfNotes);
    }
    /**
     * @deprecated Use static methods instead
     */
    add(key, ref) {
        return Trie.add(this, key, ref);
    }
    /**
     * @deprecated Use static methods instead
     */
    search(query) {
        return Trie.search(this, query);
    }
}
exports.Trie = Trie;
class TrieNode {
    constructor(key = '\0') {
        this.notes = [];
        this.children = {};
        this.key = key;
    }
    static getAllFrom(node) {
        return [
            ...node.notes,
            ...(Object.values(node.children))
                .reduce((acc, val) => acc.concat(val.getAllFrom()), [])
        ];
    }
    getAllFrom() {
        return TrieNode.getAllFrom(this);
    }
}
//# sourceMappingURL=SearchIndex.js.map