"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Translators = void 0;
const index_1 = require("./index");
const date_fns_1 = require("date-fns");
const xml2js_1 = require("xml2js");
const turndown_plugin_gfm_1 = require("turndown-plugin-gfm");
const turndown_1 = __importDefault(require("turndown"));
const crypto_1 = require("./crypto");
const date_formats_1 = require("./date-formats");
var Translators;
(function (Translators) {
    let Json;
    (function (Json) {
        /**
         * @param {string | object} json A {@link Notepad} object in JSON format or as a plain object
         * @param {string | undefined} passkey The passkey to decrypt the notepad if it's encrypted.
         * @returns {Notepad}
         */
        async function toNotepadFromNotepad(json, passkey) {
            const jsonObj = (typeof json === 'string') ? JSON.parse(json) : json;
            let notepad = new index_1.Notepad(jsonObj.title, {
                lastModified: date_fns_1.parse(jsonObj.lastModified, date_formats_1.LAST_MODIFIED_FORMAT, new Date()),
                notepadAssets: jsonObj.notepadAssets || [],
                crypto: jsonObj.crypto
            });
            if (typeof jsonObj.sections === 'string') {
                if (!passkey)
                    throw new Error('This notepad is encrypted. A passkey is needed to unlock it.');
                return await crypto_1.decrypt(jsonObj, passkey);
            }
            // Restore sections
            jsonObj.sections.forEach(section => notepad = notepad.addSection(restoreSection(section)));
            return notepad;
            function restoreSection(section) {
                let restored = new index_1.Section(section.title).clone({ internalRef: section.internalRef });
                section.sections.forEach(s => restored = restored.addSection(restoreSection(s)));
                section.notes.forEach(n => restored = restored.addNote(restoreNote(n)));
                return restored;
            }
            function restoreNote(note) {
                return new index_1.Note(note.title, note.time, note.elements, note.bibliography, note.internalRef);
            }
        }
        Json.toNotepadFromNotepad = toNotepadFromNotepad;
        /**
         * @param {string | object} json A {@link Notepad} object in JSON format or as a plain object
         * @param {string | undefined} passkey The passkey to decrypt the notepad if it's encrypted.
         * @returns {FlatNotepad}
         */
        async function toFlatNotepadFromNotepad(json, passkey) {
            return (await toNotepadFromNotepad(json, passkey)).flatten();
        }
        Json.toFlatNotepadFromNotepad = toFlatNotepadFromNotepad;
        /**
         * @param {string} json A Jupyter notebook (.ipynb)
         * @returns {string} A Markdown translation of that notebook
         */
        function toMarkdownFromJupyter(json) {
            const np = JSON.parse(json);
            let mdString = '';
            np.cells.forEach(cell => {
                if (cell.cell_type === 'markdown')
                    cell.source.forEach(line => mdString += line + '\n');
                if (cell.cell_type === 'code') {
                    mdString += '\n```\n';
                    cell.source.forEach(line => mdString += line + '\n');
                    cell.outputs.forEach(output => {
                        if (!output.text)
                            return;
                        mdString += '\n--------------------\n';
                        mdString += 'Output:\n';
                        output.text.forEach(t => mdString += t);
                        mdString += '\n--------------------\n';
                    });
                    mdString += '```\n';
                }
            });
            return mdString;
        }
        Json.toMarkdownFromJupyter = toMarkdownFromJupyter;
    })(Json = Translators.Json || (Translators.Json = {}));
    let Xml;
    (function (Xml) {
        /**
         * @param {string} xml The NPX file's contents as a string
         * @returns {Promise<Notepad>}
         */
        async function toNotepadFromNpx(xml) {
            const res = await parseXml(xml);
            const lastModifiedStr = res.notepad.$.lastModified;
            let notepad = new index_1.Notepad(res.notepad.$.title, { lastModified: lastModifiedStr ? date_fns_1.parse(lastModifiedStr, date_formats_1.LAST_MODIFIED_FORMAT, new Date()) : new Date() });
            // Parse sections/notes
            if (res.notepad.section) {
                res.notepad.section.forEach(s => notepad = notepad.addSection(parseSection(s)));
            }
            // Parse assets
            if (res.notepad.assets) {
                const assets = await Promise.all(((res.notepad.assets[0] || {}).asset || []).map(async (item) => {
                    try {
                        return new index_1.Asset(await dataURItoBlob(item._), item.$.uuid);
                    }
                    catch (e) {
                        console.warn(`Can't parse the asset ${item.$.uuid}\n${e}`);
                        return null;
                    }
                }));
                assets
                    .filter(asset => asset !== null)
                    .forEach(asset => notepad = notepad.addAsset(asset));
            }
            // Convert inline assets to full-assets
            const flatNotepad = notepad.flatten();
            const convertedAssets = [];
            const convertedNotes = await Promise.all(Object.values(flatNotepad.notes)
                .map(async (note) => {
                let elements = [];
                // Import inline assets
                for (const element of note.elements) {
                    // If it's not a binary asset with inline base64, skip
                    if (element.type === 'markdown' || element.content === 'AS') {
                        elements.push(element);
                        continue;
                    }
                    try {
                        const asset = new index_1.Asset(await dataURItoBlob(element.content));
                        elements.push(Object.assign(Object.assign({}, element), { content: 'AS', args: Object.assign(Object.assign({}, element.args), { ext: asset.uuid }) }));
                        convertedAssets.push(asset);
                    }
                    catch (e) {
                        console.warn(`Can't parse asset\n${e}`);
                    }
                }
                // Update the note with the new elements
                return flatNotepad.notes[note.internalRef].clone({
                    elements
                });
            }));
            convertedNotes.forEach(note => flatNotepad.notes[note.internalRef] = note);
            return flatNotepad.toNotepad().clone({
                assets: [
                    ...notepad.assets,
                    ...convertedAssets
                ],
                notepadAssets: [
                    ...notepad.notepadAssets,
                    ...convertedAssets.map(asset => asset.uuid)
                ]
            });
            function parseSection(sectionObj) {
                let section = new index_1.Section(sectionObj.$.title);
                // Insert sub-sections recursively because notepads are trees
                (sectionObj.section || []).forEach(item => section = section.addSection(parseSection(item)));
                // Parse notes
                (sectionObj.note || []).forEach(item => {
                    const posixTimeStr = item.$.time;
                    section = section.addNote(new index_1.Note(item.$.title, date_fns_1.parseISO(posixTimeStr).getTime(), [
                        ...([
                            'markdown',
                            'drawing',
                            'image',
                            'file',
                            'recording',
                            'pdf'
                        ]
                            .map(type => (item[type] || []).map(e => {
                            return {
                                type: type,
                                args: Object.assign(Object.assign({}, e.$), { canOptimise: e.$.canOptimise && (e.$.canOptimise.toLowerCase() == 'true') }),
                                content: e._ || ''
                            };
                        }))).reduce((acc, element) => acc.concat(element))
                    ], [
                        ...(item.bibliography[0].source || []).map(s => {
                            return {
                                id: s.$.id,
                                item: s.$.item,
                                content: s._
                            };
                        })
                    ]));
                });
                return section;
            }
        }
        Xml.toNotepadFromNpx = toNotepadFromNpx;
        /**
         * @param {string} xml The exported Evernote notepad's XML as a string
         * @returns {Promise<Notepad>}
         */
        async function toNotepadFromEnex(xml) {
            const res = await parseXml(xml, { trim: true, normalize: false });
            const exported = res['en-export'];
            let notepad = new index_1.Notepad(`${exported.$.application} Import ${date_fns_1.format(date_fns_1.parse(exported.$['export-date'], date_formats_1.ENEX_FORMAT, new Date()), date_formats_1.IMPORTED_EVERNOTE_NOTEBOOK_TIME)}`);
            let section = new index_1.Section('Imported Notes');
            (await Promise.all((exported.note || [])
                .map(async (enexNote) => {
                let note = new index_1.Note((enexNote.title || ['Imported Note'])[0], date_fns_1.parse(enexNote.created[0], date_formats_1.ENEX_FORMAT, new Date()).getTime(), [
                    // Add the general note content (text/to-do)
                    {
                        type: 'markdown',
                        args: {
                            id: 'markdown1',
                            x: '10px',
                            y: '10px',
                            width: '600px',
                            height: 'auto',
                            fontSize: '16px'
                        },
                        content: enmlToMarkdown(enexNote.content[0])
                    }
                ]);
                let fileCount = 0;
                let imageCount = 0;
                await Promise.all((enexNote.resource || []).reverse().map(async (resource) => {
                    const asset = new index_1.Asset(await dataURItoBlob(`data:${resource.mime};base64,${resource.data[0]._.replace(/\r?\n|\r/g, '')}`));
                    notepad = notepad.addAsset(asset);
                    const y = 10 + (335 * (fileCount + imageCount));
                    if (resource.mime[0].includes('image')) {
                        note = note.addElement({
                            type: 'image',
                            args: {
                                id: `image${++imageCount}`,
                                x: '650px',
                                y: y + 'px',
                                width: 'auto',
                                height: '300px',
                                ext: asset.uuid
                            },
                            content: 'AS'
                        });
                    }
                    else {
                        // Add the resource as a file
                        let filename;
                        try {
                            if (resource['resource-attributes'][0]['file-name']) {
                                filename = resource['resource-attributes'][0]['file-name'][0];
                            }
                            else if (resource['resource-attributes'][0]['source-url']) {
                                filename = resource['resource-attributes'][0]['source-url'][0].split('/').pop();
                            }
                            else {
                                filename = `file${fileCount}.${resource.mime[0].split('/').pop()}`;
                            }
                        }
                        catch (e) {
                            filename = 'imported-file' + fileCount;
                        }
                        note = note.addElement({
                            type: 'file',
                            args: {
                                id: `file${++fileCount}`,
                                x: '650px',
                                y: y + 'px',
                                width: 'auto',
                                height: 'auto',
                                ext: asset.uuid,
                                filename
                            },
                            content: 'AS'
                        });
                    }
                }));
                return note;
            })))
                .forEach(note => section = section.addNote(note));
            return notepad.addSection(section);
            function enmlToMarkdown(enml) {
                // Init Turndown service
                const service = new turndown_1.default();
                service.use(turndown_plugin_gfm_1.gfm);
                // Setup rules, these are like converters from the old to-markdown lib
                service.addRule('en-media', {
                    filter: 'en-media',
                    replacement: () => {
                        return '`there was an attachment here`';
                    }
                });
                service.addRule('crypt', {
                    filter: 'en-crypt',
                    replacement: () => '`there was encrypted text here`'
                });
                service.addRule('todo', {
                    filter: 'en-todo',
                    replacement: (content, node) => `- [${(node.getAttributeNode('checked') && node.getAttributeNode('checked').value === 'true') ? 'x' : ' '}] ${content}`
                });
                // Clean input
                let lines = enml.split('\n');
                lines = lines.slice(3, lines.length - 1);
                const html = lines
                    .map(line => line.trim())
                    .join('\n')
                    .replace(/^<en-media.*\/>$/gmi, tag => `${tag.substr(0, tag.length - 2)}>t</en-media>`);
                // Convert to markdown
                return service.turndown(html);
            }
        }
        Xml.toNotepadFromEnex = toNotepadFromEnex;
        function parseXml(xml, opts = {}) {
            return new Promise((resolve, reject) => {
                xml2js_1.parseString(xml, Object.assign({ trim: true }, opts), (err, res) => {
                    if (err)
                        reject(err);
                    resolve(res);
                });
            });
        }
    })(Xml = Translators.Xml || (Translators.Xml = {}));
    let Markdown;
    (function (Markdown) {
        /**
         * @param {Array<MarkdownImport>} markdown A list of all the markdown to be imported into notes
         */
        function toNotepadFromMarkdown(markdown) {
            let importCounter = 0;
            const section = markdown
                .map(({ title, content }) => new index_1.Note(`${title} (Import ${++importCounter})`).clone({
                elements: [
                    {
                        type: 'markdown',
                        content,
                        args: { id: 'markdown1', x: '10px', y: '10px', width: '550px', height: 'auto', fontSize: '16px' }
                    }
                ]
            }))
                .reduce((section, note) => section.addNote(note), new index_1.Section('Imported Notes'));
            return new index_1.Notepad('Markdown Import ' + date_fns_1.format(new Date(), date_formats_1.IMPORTED_MARKDOWN_NOTE_TIME)).addSection(section);
        }
        Markdown.toNotepadFromMarkdown = toNotepadFromMarkdown;
    })(Markdown = Translators.Markdown || (Translators.Markdown = {}));
    // Thanks to http://stackoverflow.com/a/12300351/998467
    async function dataURItoBlob(dataURI) {
        // convert base64 to raw binary data held in a string
        // doesn't handle URLEncoded DataURIs - see SO answer #6850276 for code that does this
        let byteString = atob(dataURI.split(',')[1]);
        // separate out the mime component
        let mimeString = dataURI.split(',')[0].split(':')[1].split(';')[0];
        // write the bytes of the string to an ArrayBuffer
        let ab = new ArrayBuffer(byteString.length);
        let ia = new Uint8Array(ab);
        for (let i = 0; i < byteString.length; i++) {
            ia[i] = byteString.charCodeAt(i);
        }
        // write the ArrayBuffer to a blob, and you're done
        return new Blob([ia], { type: mimeString });
    }
})(Translators = exports.Translators || (exports.Translators = {}));
//# sourceMappingURL=Translators.js.map